# CREATE_APPLICATION(MyApplication
# 	SOURCE_FILES   *.cc
# 	SOURCE_FILES   *.mm
# 	RESOURCES      resources/*
# 	LINK_WITH      Find
# 	COPY_FRAMEWORK Find
# )

FUNCTION(CREATE_APPLICATION)
	LIST(GET ARGN 0 APPLICATION_NAME)
	LIST(REMOVE_AT ARGN 0)

	SET(SRC_FILES        "")
	SET(RESOURCES        "")
	SET(LINK_WITH        "OakDebug")
	SET(COPY_FRAMEWORKS  "OakDebug")

	WHILE(ARGN)
		LIST(GET ARGN 0 TYPE)
		LIST(REMOVE_AT ARGN 0)

		IF("${TYPE}" STREQUAL "SOURCE_FILES")
			LIST(GET ARGN 0 ARG)
			LIST(REMOVE_AT ARGN 0)
			FILE(GLOB TMP "${ARG}")
			LIST(APPEND SRC_FILES ${TMP})
		ELSEIF("${TYPE}" STREQUAL "RESOURCES")
			LIST(GET ARGN 0 ARG)
			LIST(REMOVE_AT ARGN 0)
			FILE(GLOB TMP "${ARG}")
			LIST(APPEND RESOURCES ${TMP})
		ELSEIF("${TYPE}" STREQUAL "LINK_WITH")
			LIST(GET ARGN 0 ARG)
			LIST(REMOVE_AT ARGN 0)
			LIST(APPEND LINK_WITH "${ARG}")
		ELSEIF("${TYPE}" STREQUAL "COPY_FRAMEWORK")
			LIST(GET ARGN 0 ARG)
			LIST(REMOVE_AT ARGN 0)
			LIST(APPEND COPY_FRAMEWORKS "${ARG}")
		ELSE()
			MESSAGE("Unknown type: ${TYPE}")
			RETURN()
		ENDIF()
	ENDWHILE()

	# ============================
	# = Work With Collected Info =
	# ============================

	PROJECT("${APPLICATION_NAME}")

	FOREACH(SRC ${SRC_FILES})
		IF("${SRC}" MATCHES "\\.mm$")
			SET_SOURCE_FILES_PROPERTIES("${SRC}" PROPERTIES COMPILE_FLAGS "-fobjc-call-cxx-cdtors")
		ENDIF()
	ENDFOREACH()

	PROCESS_RESOURCES("${RESOURCES}" "Resources" ADDED_RESOURCES)

	ADD_EXECUTABLE("${APPLICATION_NAME}" MACOSX_BUNDLE ${SRC_FILES} ${ADDED_RESOURCES})
	TARGET_LINK_LIBRARIES("${APPLICATION_NAME}" ${LINK_WITH})
	ADD_DEPENDENCIES("${APPLICATION_NAME}" PCH)

	SET_TARGET_PROPERTIES("${APPLICATION_NAME}" PROPERTIES
		COMPILE_FLAGS               "-include ${PCH_BINARY_DIR}/prelude"
		MACOSX_BUNDLE_INFO_PLIST    "${${APPLICATION_NAME}_SOURCE_DIR}/resources/Info.plist"
	)

	# ===================
	# = Copy Frameworks =
	# ===================

	SET(ALL_FRAMEWORKS "")
	SET(DST "${${APPLICATION_NAME}_BINARY_DIR}/${APPLICATION_NAME}.app/Contents/Frameworks")

	FOREACH(FRAMEWORK ${COPY_FRAMEWORKS})
		IF(EXISTS "${FRAMEWORK}")
			SET(SRC "${FRAMEWORK}")
			GET_FILENAME_COMPONENT(FRAMEWORK "${FRAMEWORK}" NAME_WE)
			SET(PATH "${DST}/${FRAMEWORK}.framework")
			ADD_CUSTOM_COMMAND(OUTPUT "${PATH}"
				COMMAND mkdir -p "${DST}" && rm -rf "${PATH}" && cp -fpR "${SRC}" "${PATH}"
				COMMENT "Copying ${FRAMEWORK}.framework to ${APPLICATION_NAME}.app/Contents/Frameworks"
			)
		ELSE()
			SET(SRC "${${FRAMEWORK}_BINARY_DIR}/${FRAMEWORK}.framework")
			SET(PATH "${DST}/${FRAMEWORK}.framework")
			ADD_CUSTOM_COMMAND(OUTPUT "${PATH}"
				COMMAND mkdir -p "${DST}" && rm -rf "${PATH}" && cp -fpR "${SRC}" "${PATH}"
				DEPENDS "${FRAMEWORK}"
				COMMENT "Copying ${FRAMEWORK}.framework to ${APPLICATION_NAME}.app/Contents/Frameworks"
			)
		ENDIF()
		LIST(APPEND ALL_FRAMEWORKS "${PATH}")
	ENDFOREACH()

	ADD_CUSTOM_TARGET("${APPLICATION_NAME}CopyFrameworks" COMMAND true DEPENDS ${ALL_FRAMEWORKS})
	ADD_DEPENDENCIES("${APPLICATION_NAME}" "${APPLICATION_NAME}CopyFrameworks")

	# ================================
	# = Create Run and Debug Targets =
	# ================================

	ADD_CUSTOM_TARGET(Run${APPLICATION_NAME}
		"${Master_SOURCE_DIR}/Scripts/relaunch" "${APPLICATION_NAME}" "${${APPLICATION_NAME}_BINARY_DIR}/${APPLICATION_NAME}.app/Contents/MacOS"
		COMMENT "Running ${APPLICATION_NAME}…"
	)

	ADD_CUSTOM_TARGET(Debug${APPLICATION_NAME}
		gdb "${${APPLICATION_NAME}_BINARY_DIR}/${APPLICATION_NAME}.app/Contents/MacOS/Avian"
		COMMENT "Debugging ${APPLICATION_NAME}…"
	)

	ADD_DEPENDENCIES("Run${APPLICATION_NAME}" "${APPLICATION_NAME}")
	ADD_DEPENDENCIES("Debug${APPLICATION_NAME}" "${APPLICATION_NAME}")
ENDFUNCTION()
