# CREATE_FRAMEWORK(MyFramework
# 	SOURCE_FILES   *.cc
# 	SOURCE_FILES   *.mm
# 	PUBLIC_HEADERS MyFramework.h
# 	RESOURCES      resources/*
# 	LINK_WITH      Find
# )

FUNCTION(CREATE_FRAMEWORK)
	LIST(GET ARGN 0 FRAMEWORK_NAME)
	LIST(REMOVE_AT ARGN 0)

	SET(SRC_FILES   "")
	SET(G_FILES     "")
	SET(PUB_HEADERS "")
	SET(RESOURCES   "")
	SET(LINK_WITH   "OakDebug")

	WHILE(ARGN)
		LIST(GET ARGN 0 TYPE)
		LIST(REMOVE_AT ARGN 0)

		IF("${TYPE}" STREQUAL "SOURCE_FILES")
			LIST(GET ARGN 0 ARG)
			LIST(REMOVE_AT ARGN 0)
			FILE(GLOB TMP "${ARG}")
			LIST(APPEND SRC_FILES ${TMP})
		ELSEIF("${TYPE}" STREQUAL "ANTLR_FILES")
			LIST(GET ARGN 0 ARG)
			LIST(REMOVE_AT ARGN 0)
			FILE(GLOB TMP "${ARG}")
			LIST(APPEND G_FILES ${TMP})
		ELSEIF("${TYPE}" STREQUAL "PUBLIC_HEADERS")
			LIST(GET ARGN 0 ARG)
			LIST(REMOVE_AT ARGN 0)
			FILE(GLOB TMP "${ARG}")
			LIST(APPEND PUB_HEADERS ${TMP})
		ELSEIF("${TYPE}" STREQUAL "RESOURCES")
			LIST(GET ARGN 0 ARG)
			LIST(REMOVE_AT ARGN 0)
			FILE(GLOB TMP "${ARG}")
			LIST(APPEND RESOURCES ${TMP})
		ELSEIF("${TYPE}" STREQUAL "LINK_WITH")
			LIST(GET ARGN 0 ARG)
			LIST(REMOVE_AT ARGN 0)
			LIST(APPEND LINK_WITH ${ARG})
		ELSE()
			MESSAGE("Unknown type: ${TYPE}")
			RETURN()
		ENDIF()
	ENDWHILE()

	# ============================
	# = Work With Collected Info =
	# ============================

	PROJECT("${FRAMEWORK_NAME}")

	PROCESS_ANTLR("${G_FILES}" TMP)
	LIST(APPEND SRC_FILES ${TMP})

	FOREACH(SRC ${SRC_FILES})
		IF("${SRC}" MATCHES "\\.mm$")
			SET_SOURCE_FILES_PROPERTIES("${SRC}" PROPERTIES COMPILE_FLAGS "-fobjc-call-cxx-cdtors")
		ENDIF()
	ENDFOREACH()

	PROCESS_RESOURCES("${RESOURCES}" "Resources" ADDED_RESOURCES)

	ADD_LIBRARY("${FRAMEWORK_NAME}" SHARED ${SRC_FILES} ${PUB_HEADERS} ${ADDED_RESOURCES})
	TARGET_LINK_LIBRARIES("${FRAMEWORK_NAME}" ${LINK_WITH})
	ADD_DEPENDENCIES("${FRAMEWORK_NAME}" PCH)

	SET(SYM_FILE "${CMAKE_CURRENT_SOURCE_DIR}/exported_symbols.txt")
	IF(EXISTS "${SYM_FILE}")
		SET(VISIBILITY_FLAG "-exported_symbols_list ${SYM_FILE}")
	ELSE()
		SET(VISIBILITY_FLAG "-fvisibility=hidden") # we also set this as a compiler flag, so might be redundant
	ENDIF()

	SET_TARGET_PROPERTIES("${FRAMEWORK_NAME}" PROPERTIES
		FRAMEWORK                   TRUE
		BUILD_WITH_INSTALL_RPATH    TRUE
		INSTALL_NAME_DIR            "@executable_path/../Frameworks"
		PUBLIC_HEADER               "${PUB_HEADERS}"
		MACOSX_FRAMEWORK_INFO_PLIST "${${FRAMEWORK_NAME}_SOURCE_DIR}/resources/Info.plist"
		COMPILE_FLAGS               "-include ${PCH_BINARY_DIR}/prelude"
		LINK_FLAGS                  "${VISIBILITY_FLAG} -Wl,-dead_strip"
	)

	# ========================
	# = Symbolic Link Target =
	# ========================

	SET(DST "${PublicIncludes_BINARY_DIR}/${FRAMEWORK_NAME}")
	FOREACH(SRC ${PUB_HEADERS})
		GET_FILENAME_COMPONENT(NAME "${SRC}" NAME)
		ADD_CUSTOM_COMMAND(TARGET "${FRAMEWORK_NAME}" POST_BUILD
			COMMAND [[ -e "${DST}/${NAME}" ]] || ( mkdir -p "${DST}" && ln -s "${SRC}" "${DST}/${NAME}" )
			COMMENT "Creating link to public header ${FRAMEWORK_NAME}/${NAME}"
		)
	ENDFOREACH()
ENDFUNCTION()
